# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: release_policy.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x14release_policy.proto\x12\x12release_policy_api\"\'\n\x11\x44\x63rComputeContext\x12\x12\n\ndataRoomId\x18\x01 \x01(\t\"U\n\x15\x41llowedComputeContext\x12\x34\n\x03\x64\x63r\x18\x01 \x01(\x0b\x32%.release_policy_api.DcrComputeContextH\x00\x42\x06\n\x04kind\"\xc7\x01\n\x0fReleasePolicyV0\x12\x36\n\x0csecretSource\x18\x01 \x01(\x0b\x32 .release_policy_api.SecretSource\x12H\n\x15\x61llowedComputeContext\x18\x02 \x01(\x0b\x32).release_policy_api.AllowedComputeContext\x12\x32\n\ndataSource\x18\x03 \x01(\x0b\x32\x1e.release_policy_api.DataSource\"M\n\rReleasePolicy\x12\x31\n\x02v0\x18\x01 \x01(\x0b\x32#.release_policy_api.ReleasePolicyV0H\x00\x42\t\n\x07version\"?\n\x1a\x43omputationResultReference\x12\r\n\x05jobId\x18\x01 \x01(\t\x12\x12\n\nresultHash\x18\x02 \x01(\t\"/\n\x0cSecretSource\x12\x17\n\rsecretStoreId\x18\x01 \x01(\tH\x00\x42\x06\n\x04kind\"\x94\x01\n\nDataSource\x12\x38\n\x07\x64\x61taset\x18\x01 \x01(\x0b\x32%.release_policy_api.DatasetDataSourceH\x00\x12\x44\n\rcomputeResult\x18\x02 \x01(\x0b\x32+.release_policy_api.ComputeResultDataSourceH\x00\x42\x06\n\x04kind\")\n\x11\x44\x61tasetDataSource\x12\x14\n\x0cmanifestHash\x18\x01 \x01(\t\"-\n\x17\x43omputeResultDataSource\x12\x12\n\nresultHash\x18\x01 \x01(\tb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'release_policy_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  DESCRIPTOR._options = None
  _globals['_DCRCOMPUTECONTEXT']._serialized_start=44
  _globals['_DCRCOMPUTECONTEXT']._serialized_end=83
  _globals['_ALLOWEDCOMPUTECONTEXT']._serialized_start=85
  _globals['_ALLOWEDCOMPUTECONTEXT']._serialized_end=170
  _globals['_RELEASEPOLICYV0']._serialized_start=173
  _globals['_RELEASEPOLICYV0']._serialized_end=372
  _globals['_RELEASEPOLICY']._serialized_start=374
  _globals['_RELEASEPOLICY']._serialized_end=451
  _globals['_COMPUTATIONRESULTREFERENCE']._serialized_start=453
  _globals['_COMPUTATIONRESULTREFERENCE']._serialized_end=516
  _globals['_SECRETSOURCE']._serialized_start=518
  _globals['_SECRETSOURCE']._serialized_end=565
  _globals['_DATASOURCE']._serialized_start=568
  _globals['_DATASOURCE']._serialized_end=716
  _globals['_DATASETDATASOURCE']._serialized_start=718
  _globals['_DATASETDATASOURCE']._serialized_end=759
  _globals['_COMPUTERESULTDATASOURCE']._serialized_start=761
  _globals['_COMPUTERESULTDATASOURCE']._serialized_end=806
# @@protoc_insertion_point(module_scope)
